<?php
/*
 * @component Tablebooking
 * @website : ionutlupu.me
 * @copyright  Ionut Lupu. All rights reserved.
 * @license : http://www.gnu.org/copyleft/gpl.html GNU/GPL , see license.txt
 */
 
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

class TbSearch
{
    
    protected $restaurant;
    protected $day;
    protected $start;
    protected $end;
    protected $places = 0;
    protected $conditions;
    public $error = null;
    protected $bookingID = 0;

    public function __construct(TbRestaurant $restaurant, TbDay $day, $start, $end, $places = 1)
    {
        $this->restaurant = $restaurant;
        $this->day = $day;
        $this->start = $start;
        $this->end = $end;
        $this->places = $places;
        $this->conditions = $this->restaurant->getParams();
    }


    /**
     * 
     * method to return all available tables
     * 
     * @param  boolean $combine [ set this to true if we can combine more tables to have available the places ]
     * @return [array]
     */
    public function getAvailableTables( $combine = false )
    {
        $alltables = $this->restaurant->getTables();
        $booked = $this->getBookedTables(); 
        $possible = array();

        foreach ( $alltables as $key=>$table ) 
        {
            if (!in_array($table->id, $booked))
            {
                if ( $table->min_places <= $this->places && $this->places <= $table->max_places )
                {
                    $possible[] = $table;
                    unset($alltables[$key]);
                } 
                elseif ( $table->min_places > $this->places ) 
                {
                    unset($alltables[$key]);
                }
            } 
            else
            {
                unset($alltables[$key]);
            }
        }

        $possible = $this->sortTables( $possible, 'asc' );

        // if remain some tables that do not meet places requirement and we can combine them, let's see if it's possible
        if( $combine && !empty($alltables) ) 
        {
            $alltables = $this->sortTables( $alltables );
            $combinations = $this->getTableCombinations( $alltables );
            if (!empty($combinations))
            {
                $possible = array_merge( $possible, $combinations );
            }
        }

        return $possible;
    }

    

    /**
     *
     * method to validate search data agains different options
     * 
     * @return boolean true if all requirements are met, false if otherwise
     */
    public function validateInput()
    {
        if ( $this->start == $this->end || $this->start > $this->end )
        {
            $this->error = JText::_('COM_TABLEBOOKING_SEARCH_ERROR_WRONG_INTERVALS');
        }

        if ( $this->places == 0 )
        {
            $this->error = JText::_('COM_TABLEBOOKING_SEARCH_ERROR_WRONG_PLACES');
        }

        if (!empty($this->conditions->minplaces) && $this->places < $this->conditions->minplaces) 
        {
            $this->error = sprintf(JText::_('COM_TABLEBOOKING_SEARCH_ERROR_MIN_PLACES'), $this->conditions->minplaces);
        }


        if (!empty($this->conditions->maxplaces) && $this->places > $this->conditions->maxplaces) 
        {
            $this->error = sprintf(JText::_('COM_TABLEBOOKING_SEARCH_ERROR_MAX_PLACES'), $this->conditions->maxplaces);
        }

 
        $tz = new DateTimeZone(JFactory::getApplication()->getCfg('offset'));
        $now = JFactory::getDate('now',$tz);
		
		// if selected day is today
        if (TbDisplay::strToTime($this->day->date->format('Y-m-d')) == TbDisplay::strToTime($now->format('Y-m-d', true))) 
        {	
			$current = $now->format('H',true) * 3600 + $now->format('i',true) * 60;
			if ($current > $this->start)
			{
				$this->error = JText::_('COM_TABLEBOOKING_SEARCH_ERROR_WRONG_DATETIME');
			}
        }
        // if in the past
        elseif (TbDisplay::strToTime($this->day->date->format('Y-m-d')) < TbDisplay::strToTime($now->format('Y-m-d', true)))
        {
			$this->error = JText::_('COM_TABLEBOOKING_SEARCH_ERROR_WRONG_DATETIME');
		}

        return $this->error ? false : true;
    }


    /**
     * 
     * method to return an array with all table id that are booked
     * @return array
     */
    protected function getBookedTables() 
    {
        $booked = array();
        $db = JFactory::getDbo();
        $query = $db->getQuery(true);
        $query->select( $db->quoteName('table') );
        $query->from( $db->quoteName('#__tbb_bookings') );
        $query->where( $db->quoteName('restaurant') . ' = ' . $this->restaurant->id );
        $query->where( $db->quoteName('bookdate') . ' = ' . $db->quote($db->escape($this->day->date->format('Y-m-d'))) );
        $query->where('(  
                        (' . $db->quoteName('bookstart') . ' < ' . $this->start . ' AND ' . $db->quoteName('bookend') . ' > ' . $this->start . ' ) OR 
                        (' . $db->quoteName('bookstart') . ' >= ' . $this->start . ' AND ' . $db->quoteName('bookend') . ' <= ' . $this->end . ') OR
                        (' . $db->quoteName('bookstart') . ' >= ' . $this->start . ' AND ' . $db->quoteName('bookstart') . ' < ' . $this->end . ' AND ' . $db->quoteName('bookend') . ' > ' . $this->end . ') 
                        )');
        $query->where('status != 2');
        
        // if we want to search for an existing booking, free again the tables booked for this reservation
        if ( $this->bookingID ) 
        {
            $query->where( $db->quoteName('id') . ' != ' . $this->bookingID );
        }

        $db->setQuery( $query );
        $result = $db->loadColumn();

        if (!empty($result))
        {
            foreach ( $result as $value ) 
            {
                $tables = array_values( json_decode( $value ) );
                $booked = array_merge( $booked, $tables );
            }
        }

        return $booked;
    }
    

 
    /**
     * method to get possible table combinations
     * @param  [array] $tables [remaining tables after we filter tables that meet place requirements]
     * @return [array]        
     */
    protected function getTableCombinations( $tables )
    {   

        $end = false;
        $possible = array();
        while ( $end == false )
        {
            $total = 0;
            $combination = array();
            foreach ($tables as $key=>$table)
            {
                if ( $total < $this->places ) 
                {
                    $combination[] = $table;
                    $total += $table->max_places;
                    unset($tables[$key]);
                }
            }
            if ( $total < $this->places ) 
            {
                $end = true;
            } 
            else 
            {
                $possible[] = $combination;
            }
        }
        return $possible;
    }


    /**
     * sort tables depending on the algorithm ( desc = descending, asc = ascending )
     * @param  [array] $tables [tables to be sort]
     * @param  [string] $direction - desc (from higher places to smaller), asc (from smaller places to higher) 
     * @return [array]         sorted tables
     */
    protected function sortTables( $tables, $direction = 'desc' ) 
    {

        foreach( $tables as $key=>$table )
        {
            // if we have a next table
            if (isset($tables[$key+1])) 
            {
                if (($table->max_places < $tables[$key+1]->max_places && $direction == 'desc') || ($table->max_places > $tables[$key+1]->max_places && $direction == 'asc')) 
                {
                    $first = $tables[$key+1];
                    $next  = $table;
                    $tables[$key] = $first;
                    $tables[$key+1] = $next;
                    return $this->sortTables( $tables, $direction );
                }
            }
        }
        return $tables;
    }


    /**
     * set the booking so that we know that the search is being done for an existing booking
     * this way we'll free the tables that are already booked
     * @param [int] $bookingID [the booking ID that we want to modify details]
     */
    public function setBooking( $bookingID )
    {
        $this->bookingID = $bookingID;
    }





}

?>
